/*
 * vfilter can understand version strings
 *
 * Copyright (C) 2006 Jean-Philippe Guillemin <jp.guillemin@free.fr> and Thibaud Guerin <guth@ploposor.com>
 *
 * This file is free software; the copyright holder gives unlimited
 * permission to copy and/or distribute it, with or without
 * modifications, as long as this notice is preserved.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, to the extent permitted by law; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.
 */

#include <EXTERN.h>
#include <perl.h>
#include "vfilter.h"

regex_t regex;
regmatch_t pmatch[3];
size_t size = 0;

/**********************/
/*      PROTOTYPES    */
/**********************/

/* Get version and build fields from package name */
int getfields (struct pkgfields *pkg);

/* Transform homosapiens into Hal9000 style version string */
int serialize (struct pkgfields *pkg);

/* string version comparison function */
int strverscmp(const char *s1, const char *s2);

/* usage function */
int usage(void);


/****** perl called function *******/

int testme(char *s1, char *s2)
{
  return (1);
}

int test_package(char *s1, char *s2)
{
  struct pkgfields * pkg1;
  struct pkgfields * pkg2;
  int	versioncmp;
  int	buildcmp;


  if ( regcomp(&regex, "^.*-([^-]*)-[^-]*-([^-]*)\\.t[glx]z$", REG_EXTENDED )) {
    dbg("getfields() : regex failed compilation\n");
    return(EXIT_FAILURE);
  }
  pkg1=(struct pkgfields *) malloc (sizeof(struct pkgfields));
  pkg2=(struct pkgfields *) malloc (sizeof(struct pkgfields));

  pkg1->name = s1;
  pkg2->name = s2;

  if ( getfields (pkg1) || getfields (pkg2)
       || serialize (pkg1) || serialize (pkg2)) {
    dbg("\n--ZW::Netpkg::PVfilter::test_package(s1,s2): errror, return -42\n");
    return(-42);
  }

  versioncmp=strcmp(pkg1->vserial, pkg2->vserial);

  if ( versioncmp < 0){
    // version1 < version2 <=> downgrade
    return (-1);
  }else if( versioncmp > 0){
    //version1 > version2 <=> update
    return (1);
  }

  /* if we are here then version1 == version2, so we need to
     take a look at the BUILD field */
  buildcmp=strcmp(pkg1->bserial, pkg2->bserial);

  dbg("\n--ZW::Netpkg::PVfilter::test_package(s1,s2): Info --buildcmp : %i\n - s1:%s\n - s2:%s\n - bserial1:%s\n - bserial2:%s\n", buildcmp, s1, s2, pkg1->bserial, pkg2->bserial);
  if (! buildcmp)
    // build1 == version2 <=> same package
    return (0);
  else if ( buildcmp < 0) {
    // build1 < build2 <=> downgrade
    return (-1);
  }else {
    // build1 > build2 <=> update
    return (1);
  }
  // build1 == version2 <=> same package
  return (0);
}


/************* helper functions *****************/
int getfields (struct pkgfields *pkg) {

  int start = 0;
  int end = 0;

  if ( regexec(&regex, pkg->name, 3, pmatch, 0) ) {
    dbg("getfields() : regex doesn't match\n");
    return(EXIT_FAILURE);
  } else{
    // Retrieving the version
    start = pmatch[1].rm_so;
    end = pmatch[1].rm_eo;
    size = end - start;
    strncpy (pkg->version, pkg->name + start, size);
    pkg->version[size] = '\0';

    dbg("getfields() : package version = %s\n", pkg->version);

    // Retrieving the build
    start = pmatch[2].rm_so;
    end = pmatch[2].rm_eo;
    size = end - start;
    strncpy (pkg->build, pkg->name + start, size);
    pkg->build[size] = '\0';

    dbg("getfields() : package build = %s\n", pkg->build);

  }

  return(EXIT_SUCCESS);
}


char *strings_tab[]=
  { "cvs","svn","git","alpha1","alpha2","alpha3","alpha4","alpha",
    "beta1","beta2","beta3","beta4","beta","pre1","pre2","pre3","pre4",
    "pre","rc1","rc2","rc3","rc4","rc",
    "a","b","c","d","e","f","g","h","i","j","k","l",0 };
char high_flags_tab[]="000111112222233333444446666777788880";
char low_flags_tab[]="555123451234512345123451234123412340";

/* Transform homosapiens into Hal9000 style version string */
int serialize (struct pkgfields *pkg) {

  unsigned int i = 0;
  unsigned int dots = 0;
  unsigned int found = 0;
  unsigned int index = 0;
  char flag[3];
  char *offset = 0;
  char buffer[TINY_BUFFER_SIZE];

  // Init Buffers *****************

  memset(buffer, 0, 9);
  memcpy(flag, "55", 3); // copy string + '\0'
  memset(pkg->vserial, '0', BIG_BUFFER_SIZE);
  pkg->vserial[BIG_BUFFER_SIZE] = 0;
  memset(pkg->bserial, '0', BIG_BUFFER_SIZE);
  pkg->bserial[BIG_BUFFER_SIZE] = 0;

  // Processing version *****************

  offset = pkg->version;

  // We remove leading '0'
  while ((*offset) == '0')
    offset++;

  for (; *offset; offset++) {

    /* check numbers first as they are the most common char */
    if (*offset >= '0' && *offset <= '9'){
      buffer[i++] = *(offset);
      continue;
    }

    /* check "." or "_" */
    if (*offset == '.' || *offset == '_'){
      memcpy (pkg->vserial + (++dots * MEDIUM_BUFFER_SIZE) - i, buffer, i);
      i=0;
      continue;
    }

    /* check "cvs", "svn", "git", "alpha", "beta", "pre" and "rc" ... */
    found=FALSE;
    for (index=0; strings_tab[index]; index++) {
      if ( !strncasecmp(offset, strings_tab[index], strlen(strings_tab[index])) ){
	flag[0] = high_flags_tab[index];
	flag[1] = low_flags_tab[index];
	offset += strlen(strings_tab[index]) - 1; // -1 as we offset++ before next global for loop
	found=TRUE;
	break;
      }
    }
    if ( found == FALSE )
	buffer[i++] = *(offset);
  }

  // Fill vserial and set ending flag
  memcpy (pkg->vserial + (++dots * MEDIUM_BUFFER_SIZE) - i, buffer, i);
  memcpy(pkg->vserial + BIG_BUFFER_SIZE -2, flag, 2);

  // Just to be sure
  pkg->vserial[BIG_BUFFER_SIZE] = '\0';

  dbg("serialize() : vserial = %s\n", pkg->vserial);


  // Processing build *****************

  // Re-init vars
  i=0;
  dots=0;
  offset = pkg->build;

  // We remove leading '0'
  while ((*offset) == '0')
    offset++;

  for (; *offset; offset++) {

    /* check numbers first as they are the most common char */
    if (*offset >= '0' && *offset <= '9'){
      buffer[i++] = *(offset);
      continue;
    }

    /* check "." or "_" or "z" */
    if (*offset == '.' || *offset == '_' || *offset == 'z'){
      memcpy (pkg->bserial + (++dots * MEDIUM_BUFFER_SIZE) - i, buffer, i);
      i=0;
      continue;
    }

    buffer[i++] = *(offset);
  }

  memcpy (pkg->bserial + (++dots * MEDIUM_BUFFER_SIZE) - i, buffer, i);

  // Just to be sure
  pkg->bserial[BIG_BUFFER_SIZE] = '\0';

  dbg("serialize() : bserial = %s\n", pkg->bserial);

  return(EXIT_SUCCESS);
}

